import abc


class BasePlugin(abc.ABC):

    # For every manifest format in the .yaml file, there will be a copy of
    # the plugin having the "process" function specialized for the particular
    # format.
    def onCreate(self, manifest_format, config):
        assert (hasattr(self, manifest_format))
        self.process = getattr(self, manifest_format)
        self._config = config

    # Plugins MUST reimplement at least these methods to be valid

    # This is invoked at creation time and is necessary to validate
    #  that the .yaml configuration provided for the specific plugin
    #  is valid. The recommended way to implement this is using Schema.
    # Doing it here has the advantages of having the actual manifest processing
    #  code (i.e mpd()) cleaner (you can give for granted that your config
    #  dict will have the expected keys,values). Plus, configuration validation
    #  can be computationally expensive so it is just right to do it once
    #  when the .yaml config file is loaded.

    def config(self, storage):
        # The "config" method of a BasePlugin will return a dictionary type
        if self._config is not None:
            return self._config
        else:
            # Plugins will pass a configuration down the pipeling by putting it
            #  into a {<plugin name> : {<config>}} dictionary. The split is
            #  there to remove the path. They would use the name of the plugin
            #  that must receive the configuration.
            return storage.get(self._name.split('.')[-1], {})

    @property
    @abc.abstractmethod
    def schema(self):
        raise NotImplementedError

    def validate_config(self):
        self._config = self.schema().validate(self._config)
        return self._config

    # TODO: the right thing is to find a way to have Plugin class
    #  reimplement at least one amongst the 4 possible entrypoint.
    # Their name should be taken from the 4 supported formats
    #  that are now defined in pipeline.py
    @abc.abstractmethod
    def mpd(self, manifest, storage):
        raise NotImplementedError
